@@ -165,6 +165,9 @@ gtk_accel_group_finalize (GObject *object)
 	  _gtk_accel_map_remove_group (accel_path, accel_group);
 	}
       g_closure_remove_invalidate_notifier (entry->closure, accel_group, accel_closure_invalidate);
+
+      /* remove quick_accel_add() refcount */
+      g_closure_unref (entry->closure);
     }
 
   g_free (accel_group->priv_accels);
@@ -834,7 +837,6 @@ gtk_accel_group_from_accel_closure (GClosure *closure)
  *                 to activate the accelerator.
  * @accel_key:     accelerator keyval from a key event
  * @accel_mods:    keyboard state mask from a key event
- * @returns:       %TRUE if the accelerator was handled, %FALSE otherwise
  * 
  * Finds the first accelerator in @accel_group 
  * that matches @accel_key and @accel_mods, and
@@ -867,7 +869,6 @@ gtk_accel_group_activate (GtkAccelGroup   *accel_group,
  *                 to activate the accelerator.
  * @accel_key:     accelerator keyval from a key event
  * @accel_mods:    keyboard state mask from a key event
- * @returns:       %TRUE if the accelerator was handled, %FALSE otherwise
  * 
  * Finds the first accelerator in any #GtkAccelGroup attached
  * to @object that matches @accel_key and @accel_mods, and
