@@ -305,9 +305,6 @@ test_printer_create_cairo_surface (GtkPrinter       *printer,
   else
     surface = cairo_pdf_surface_create_for_stream (_cairo_write, cache_io, width, height);
 
-  if (gtk_print_settings_get_printer_lpi (settings) == 0.0)
-    gtk_print_settings_set_printer_lpi (settings, 150.0);
-
   cairo_surface_set_fallback_resolution (surface,
                                          2.0 * gtk_print_settings_get_printer_lpi (settings),
                                          2.0 * gtk_print_settings_get_printer_lpi (settings));
